import numpy as np
from collections import defaultdict
from itertools import combinations
from gismap.lab.vis import generate_html


def initials(name):
    """
    Parameters
    ----------
    name: :class:`str`
        Person's name.

    Returns
    -------
    :class:`str`
        Person's initials (2 letters only).
    """
    first_letters = [w[0] for w in name.split()]
    return first_letters[0] + first_letters[-1]


def author_to_html(author):
    """
    Parameters
    ----------
    author: :class:`~gismap.sources.models.Author`
        Searcher.

    Returns
    -------
    HTML string with URL if applicable.
    """
    name = getattr(author, "name", "Unknown Author")
    # Try direct URL property (optional)
    url = getattr(author, "url", None)
    # For LabAuthor, check metadata.url
    if hasattr(author, "metadata"):
        meta_url = getattr(author.metadata, "url", None)
        if meta_url:
            url = meta_url
        elif hasattr(author.sources[0], "url"):
            url = author.sources[0].url
    if url:
        return f'<a href="{url.strip()}" target="_blank">{name}</a>'
    else:
        return name


def publication_to_html(pub):
    """
    Parameters
    ----------
    pub: :class:`~gismap.sources.models.Publication`
        Publication.

    Returns
    -------
    HTML string with hyperlinks where applicable.
    """
    # Title as link if available
    url = getattr(pub, "url", None)
    if url is None and hasattr(pub, "sources"):
        url = getattr(pub.sources[0], "url", None)
    if url:
        title_html = f'<a href="{url}" target="_blank">{pub.title}</a>'
    else:
        title_html = pub.title

    # Authors: render in order, separated by comma
    author_html_list = [
        author_to_html(author) for author in getattr(pub, "authors", [])
    ]
    authors_html = ", ".join(author_html_list)

    # Venue, Year
    venue = getattr(pub, "venue", "")
    year = getattr(pub, "year", "")

    # Basic HTML layout
    html = f"{title_html}, by <i>{authors_html}</i>. {venue}, {year}."
    return html.strip()


def publications_list_html(publications, n=10):
    """

    Parameters
    ----------
    publications: :class:`list` of :class:`~gismap.sources.models.Publication`
        Publications to display.
    n: :class:`int`, default=10
        Number of publications to display. If there are more publications, a *Show more* option is available to unravel them.

    Returns
    -------
    :class:`str`
    """
    list_items = []
    for i, pub in enumerate(publications):
        item = publication_to_html(pub)
        if i < n:
            li = f"<li>{item}</li>"
        else:
            li = f'<li class="extra-publication" style="display:none;">{item}</li>'
        list_items.append(li)
    ul_content = "\n".join(list_items)

    if len(publications) <= n:
        show_more_part = ""
    else:
        # Add a "Show more" link and JavaScript for toggling
        show_more_part = """
<li>
  <a href="#" onclick="
    var elts = this.parentElement.parentElement.querySelectorAll('.extra-publication');
    for (var i = 0; i < elts.length; ++i) {elts[i].style.display = 'list-item';}
    this.parentElement.style.display = 'none';
    return false;">Show more…</a>
</li>
        """

    html = f"""<ul>
{ul_content}
{show_more_part}
</ul>
"""
    return html


def to_node(s, node_pubs):
    """
    Parameters
    ----------
    s: :class:`~gismap.lab.lab_author.LabAuthor`
        Searcher.
    node_pubs: :class:`dict`
        Lab publications.

    Returns
    -------
    :class:`dict`
        A display-ready representation of the searcher.
    """
    res = {
        "id": s.key,
        "hover": f"Click for details on {s.name}.",
        "overlay": f"<div> Publications of {author_to_html(s)}:</div><div>{publications_list_html(node_pubs[s.key])}</div>",
        "group": s.metadata.group,
    }
    if s.metadata.img:
        res.update({"image": s.metadata.img, "shape": "circularImage"})
    else:
        res["label"] = initials(s.name)
    if s.metadata.position:
        x, y = s.metadata.position
        res.update({"x": x, "y": y, "fixed": True})
    return res


def to_edge(k, v, searchers):
    """
    Parameters
    ----------
    k: :class:`tuple`
        Keys of the searchers involved.
    v: :class:`list`
        List of joint publications.
    searchers: :class:`dict`
        Searchers.

    Returns
    -------
    :class:`dict`
        A display-ready representation of the collaboration edge.
    """
    strength = 1 + np.log2(len(v))
    return {
        "from": k[0],
        "to": k[1],
        "hover": f"Show joint publications from {searchers[k[0]].name} and {searchers[k[1]].name}",
        "overlay": f"<div> Joint publications from {author_to_html(searchers[k[0]])} and {author_to_html(searchers[k[1]])}:</div><div>{publications_list_html(v)}</div>",
        "width": int(strength),
        "length": int(200 / strength),
    }


def lab2graph(lab):
    """
    Parameters
    ----------
    lab: :class:`~gismap.lab.lab.Lab`
        A lab populated with searchers and publications.

    Returns
    -------
    :class:`str`
        Collaboration graph.

    Examples
    --------

    >>> from gismap.lab import ListLab
    >>> lab = ListLab(author_list=['Tixeuil Sébastien', 'Mathieu Fabien'], name='mini')
    >>> lab.update_authors()
    >>> lab.update_publis()
    >>> len(lab.authors)
    2
    >>> 400 < len(lab.publications) < 440
    True
    >>> html = lab2graph(lab)
    >>> html[:80]  # doctest: +ELLIPSIS
    '\\n<div class="gismap-content">\\n<div id="mynetwork_..."></div>\\n<a\\n  href="htt'
    """
    node_pubs = {k: [] for k in lab.authors}
    edges_dict = defaultdict(list)
    for p in lab.publications.values():
        # Strange things can happen with multiple sources. This should take care of it.
        lauths = {a.key: a for source in p.sources for a in source.authors if a.__class__.__name__ == "LabAuthor"}
        lauths = sorted([a for a in lauths.values()], key=lambda a: str(a.key))
        for a in lauths:
            node_pubs[a.key].append(p)
        for a1, a2 in combinations(lauths, 2):
            edges_dict[a1.key, a2.key].append(p)
    connected = {k for kl in edges_dict for k in kl}

    for k, v in node_pubs.items():
        node_pubs[k] = sorted(v, key=lambda p: -p.year)
    for k, v in edges_dict.items():
        edges_dict[k] = sorted(v, key=lambda p: -p.year)

    return generate_html(
        nodes=[
            to_node(s, node_pubs) for s in lab.authors.values() if s.key in connected
        ],
        edges=[to_edge(k, v, lab.authors) for k, v in edges_dict.items()],
    )



#!/usr/bin/env python
#
# gismap documentation build configuration file, created by
# sphinx-quickstart on Fri Jun  9 13:47:02 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another
# directory, add these directories to sys.path here. If the directory is
# relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
#
import os
import sys

sys.path.insert(0, os.path.abspath(".."))

import gismap

# -- General configuration ---------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.napoleon",
    "nbsphinx",
    "IPython.sphinxext.ipython_console_highlighting",
    "sphinx.ext.intersphinx",
    "sphinx.ext.imgconverter",
    "myst_parser",
    "sphinx_copybutton",
    "sphinx.ext.githubpages",
]

# Remote NB execution
nbsphinx_execute = 'always'
nbsphinx_execute_arguments = [
    "--InlineBackend.figure_formats={'svg', 'pdf'}"
]

def setup(app):
    app.add_css_file("https://unpkg.com/vis-network/styles/vis-network.min.css")
    app.add_js_file("https://unpkg.com/vis-network/standalone/umd/vis-network.min.js")


# Smart code copy
copybutton_exclude = ".linenos, .gp, .go"

# MYST configuration
myst_enable_extensions = ["linkify", "dollarmath", "colon_fence"]
myst_heading_anchors = 3
myst_links_external_new_tab = True

# Add the possibility to access python documentation.
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "sklearn": ("https://scikit-learn.org/stable", None),
    "ipython": ("https://ipython.readthedocs.io/en/stable/", None),
    "numba": ("https://numba.readthedocs.io/en/stable/", None),
    "numpy": ("https://numpy.org/doc/stable/", None),
    "scipy": ("https://docs.scipy.org/doc/scipy/", None),
    "matplotlib": ("https://matplotlib.org/stable/", None),
}

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = [".rst", ".md"]
# source_suffix = '.rst'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "Generic Information Search: Mapping and Analysis of Publications"
copyright = "2025, Fabien Mathieu"
author = "Fabien Mathieu"

# The version info for the project you're documenting, acts as replacement
# for |version| and |release|, also used in various other places throughout
# the built documents.
#
# The short X.Y version.
version = gismap.__version__
# The full version, including alpha/beta/rc tags.
release = gismap.__version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = "pydata_sphinx_theme"

html_logo = "logo.png"
html_favicon = "favicon.ico"

# Theme options are theme-specific and customize the look and feel of a
# theme further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}
html_theme_options = {
    "icon_links": [
        {
            "name": "GitHub",
            "url": "https://github.com/balouf/gismap",
            "icon": "fa-brands fa-github",
        },
    ],
    "header_links_before_dropdown": 5,
    "show_nav_level": 2,
    "show_toc_level": 2,
    "navigation_depth": 2,
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']


# -- Options for HTMLHelp output ---------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "gismapdoc"


# -- Options for LaTeX output ------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        "gismap.tex",
        "Generic Information Search: Mapping and Analysis of Publications Documentation",
        "Fabien Mathieu",
        "manual",
    ),
]


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        master_doc,
        "gismap",
        "Generic Information Search: Mapping and Analysis of Publications Documentation",
        [author],
        1,
    )
]


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "gismap",
        "Generic Information Search: Mapping and Analysis of Publications Documentation",
        author,
        "gismap",
        "One line description of project.",
        "Miscellaneous",
    ),
]



from dataclasses import dataclass
from typing import ClassVar

from gismap.utils.common import LazyRepr


@dataclass(repr=False)
class Author(LazyRepr):
    name: str


@dataclass(repr=False)
class Publication(LazyRepr):
    title: str
    authors: list
    venue: str
    type: str
    year: int


@dataclass(repr=False)
class DB(LazyRepr):
    db_name: ClassVar[str] = None

    @classmethod
    def search_author(cls, name):
        raise NotImplementedError

    @classmethod
    def from_author(cls, a):
        raise NotImplementedError



from dataclasses import dataclass, field
from bof.fuzz import Process
import numpy as np

from gismap.sources.models import Publication, Author
from gismap.utils.text import clean_aliases


def score_author_source(dbauthor):
    if dbauthor.db_name == "hal":
        if dbauthor.key_type == "fullname":
            return -1
        elif dbauthor.key_type == "pid":
            return 2
        else:
            return 3
    elif dbauthor.db_name == "dblp":
        return 1
    else:
        return 0


def sort_author_sources(sources):
    return sorted(sources, key=score_author_source, reverse=True)


@dataclass(repr=False)
class SourcedAuthor(Author):
    sources: list = field(default_factory=list)

    @property
    def key(self):
        if self.sources:
            return self.sources[0].key
        else:
            return None

    @property
    def aliases(self):
        if self.sources:
            return clean_aliases(
                self.name, [n for a in self.sources for n in [a.name] + a.aliases]
            )
        else:
            return []

    @classmethod
    def from_sources(cls, sources):
        sources = sort_author_sources(sources)
        return cls(name=sources[0].name, sources=sources)

    def get_publications(self, clean=True, selector=None):
        if selector is None:
            selector = []
        res = {
            p.key: p for a in self.sources for p in a.get_publications() if all(f(p) for f in selector)
        }
        if clean:
            regroup_authors({self.key: self}, res)
            return regroup_publications(res)
        else:
            return res


publication_score_rosetta = {
    "db_name": {"dblp": 1, "hal": 2},
    "venue": {"CoRR": -1, "unpublished": -2},
    "type": {"conference": 1, "journal": 2},
}


def score_publication_source(source):
    scores = [
        v.get(getattr(source, k, None), 0) for k, v in publication_score_rosetta.items()
    ]
    scores.append(source.year)
    return tuple(scores)


def sort_publication_sources(sources):
    return sorted(sources, key=score_publication_source, reverse=True)


@dataclass(repr=False)
class SourcedPublication(Publication):
    sources: list = field(default_factory=list)

    @property
    def key(self):
        if self.sources:
            return self.sources[0].key
        else:
            return None

    @classmethod
    def from_sources(cls, sources):
        sources = sort_publication_sources(sources)
        main = sources[0]
        res = cls(
            **{
                k: getattr(main, k)
                for k in ["title", "authors", "venue", "type", "year"]
            },
            sources=sources,
        )
        return res


def regroup_authors(auth_dict, pub_dict):
    """
    Replace authors of publications with matching authors.
    Typical use: upgrade DB-specific authors to multisource authors.

    Replacement is in place.

    Parameters
    ----------
    auth_dict: :class:`dict`
        Authors to unify.
    pub_dict: :class:`dict`
        Publications to unify.

    Returns
    -------
    None
    """
    redirection = {
        k: a
        for a in auth_dict.values()
        for s in a.sources
        for k in [s.key, s.name, *s.aliases]
    }

    for pub in pub_dict.values():
        pub.authors = [redirection.get(a.key, a) for a in pub.authors]


def regroup_publications(pub_dict, threshold=85, length_impact=0.05, n_range=5):
    """
    Puts together copies of the same publication.

    Parameters
    ----------
    pub_dict: :class:`dict`
        Publications to unify.
    threshold: float
        Similarity parameter.
    length_impact: float
        Length impact parameter.

    Returns
    -------
    :class:`dict`
        Unified publications.
    """
    pub_list = [p for p in pub_dict.values()]
    res = dict()

    if pub_list:

        p = Process(length_impact=length_impact, n_range=n_range)
        p.fit([paper.title for paper in pub_list])

        done = np.zeros(len(pub_list), dtype=bool)
        for i, paper in enumerate(pub_list):
            if done[i]:
                continue
            locs = np.where(p.transform([paper.title])[0, :] > threshold)[0]
            pub = SourcedPublication.from_sources([pub_list[i] for i in locs])
            res[pub.key] = pub
            done[locs] = True
    return res



def taboos(txt, words):
    if isinstance(words, str):
        return words not in txt
    else:
        return all(w not in txt for w in words)

def publication_size_filter(n_max=10):
    return lambda p: len(p.authors) < n_max

def publication_taboo_filter(w=None):
    if w is None:
        w = ["Editorial", "Foreword"]
    return lambda p: taboos(p.title, w)

def author_taboo_filter(w=None):
    if w is None:
        w = ["Buob", "Kofman"]
    return lambda a: taboos(a.name, w)



from gismap.lab.lab import Lab
from gismap.lab.lab_author import LabAuthor


class EgoMap(Lab):
    """
    Parameters
    ----------
    sun
    args
    kwargs

    Examples
    --------

    >>> dang = EgoMap("The-Dang Huynh", dbs="hal")
    >>> dang.build(target=10)
    >>> sorted(a.name for a in dang.authors.values())  # doctest: +NORMALIZE_WHITESPACE
    ['Bruno Kauffmann', 'Chung Shue Chen', 'Fabien Mathieu', 'François Baccelli', 'Laurent Viennot', 'Ludovic Noirie',
    'Siu-Wai Ho', 'Sébastien Tixeuil', 'The-Dang Huynh', 'Yannick Carlinet']
    """

    def __init__(self, sun, *args, **kwargs):
        if isinstance(sun, str):
            sun = LabAuthor(sun)
        sun.metadata.position = (0, 0)
        self.sun = sun
        super().__init__(*args, **kwargs)

    def _author_iterator(self):
        yield self.sun

    def build(self, **kwargs):
        target = kwargs.pop("target", 50)
        group = kwargs.pop("group", "moon")
        self.update_authors(desc="Sun metadata")
        self.update_publis(desc="Sun publications")
        kwargs["target"] = target - len(self.authors)
        self.expand(group=None, desc="Planets", **kwargs)
        kwargs.update({"target": target - len(self.authors), "group": group})
        if kwargs["target"] > 0:
            self.expand(desc="Moons", **kwargs)



from bs4 import BeautifulSoup as Soup
import re

from gismap.lab.lab import Lab
from gismap.lab.lab_author import AuthorMetadata, LabAuthor
from gismap.utils.requests import get


class Lip6Lab(Lab):
    """
    Class for handling a LIP6 team using `https://www.lip6.fr/recherche/team_membres.php?acronyme=*team_acronym*` as entry point.
    Default to `NPA` team.
    """

    name = "NPA"

    def _author_iterator(self):
        url = f"https://www.lip6.fr/recherche/team_membres.php?acronyme={self.name}"
        soup = Soup(get(url), "lxml")
        for a in soup.table("a"):
            name = a.text.replace("\xa0", " ").strip()
            if not name:
                continue
            metadata = AuthorMetadata(group=self.name)
            previous = a.find_previous_sibling()
            if previous is not None and "user" in previous.get("class", []):
                metadata.url = previous["href"].strip()
            yield LabAuthor(name=name, metadata=metadata)


class Lip6(Lip6Lab):
    """
    Class for handling all LIP6 teams using `https://www.lip6.fr/informations/annuaire.php` to get team names.
    """

    name = "LIP6"

    def _author_iterator(self):
        groups = re.compile(r'acronyme=(.*?)[\'"]')
        for group in groups.findall(
            get("https://www.lip6.fr/informations/annuaire.php")
        ):
            for author in Lip6Lab(name=group)._author_iterator():
                yield author



HIDDEN_KEYS = {"sources", "aliases", "abstract", "metadata"}


class LazyRepr:
    """
    MixIn that hides empty fields in dataclasses repr's.
    """

    def __repr__(self):
        kws = [
            f"{key}={value!r}"
            for key, value in self.__dict__.items()
            if value and key not in HIDDEN_KEYS and not key.startswith("_")
        ]
        return f"{type(self).__name__}({', '.join(kws)})"


def unlist(x):
    """
    Parameters
    ----------
    x: :class:`str` or :class:`list` or :class:`int`
        Something.

    Returns
    -------
    x: :class:`str` or :class:`int`
        If it's a list, make it flat.
    """
    return x[0] if (isinstance(x, list) and x) else x


def get_classes(root, key="name"):
    """
    Parameters
    ----------
    root: :class:`class`
        Starting class (can be abstract).
    key: :class:`str`, default='name'
        Attribute to look-up

    Returns
    -------
    :class:`dict`
        Dictionaries of all subclasses that have a key attribute (as in class attribute `key`).

    Examples
    --------

    >>> from gismap.sources.models import DB
    >>> subclasses = get_classes(DB, key='db_name')
    >>> dict(sorted(subclasses.items())) # doctest: +NORMALIZE_WHITESPACE
    {'dblp': <class 'gismap.sources.dblp.DBLP'>, 'hal': <class 'gismap.sources.hal.HAL'>}
    """
    result = {
        getattr(c, key): c for c in root.__subclasses__() if getattr(c, key, None)
    }
    for c in root.__subclasses__():
        result.update(get_classes(c))
    return result


def list_of_objects(clss, dico, default=None):
    """
    Versatile way to enter a list of objects referenced by a dico.

    Parameters
    ----------
    clss: :class:`object`
        Object or reference to an object or list of objects / references to objects.
    dico: :class:`dict`
        Dictionary of references to objects.
    default: :class:`list`, optional
        Default list to return if `clss` is None.

    Returns
    -------
    :class:`list`
        Proper list of objects.

    Examples
    ________

    >>> from gismap.sources.models import DB
    >>> subclasses = get_classes(DB, key='db_name')
    >>> from gismap import HAL, DBLP
    >>> list_of_objects([HAL, 'dblp'], subclasses)
    [<class 'gismap.sources.hal.HAL'>, <class 'gismap.sources.dblp.DBLP'>]
    >>> list_of_objects(None, subclasses, [DBLP])
    [<class 'gismap.sources.dblp.DBLP'>]
    >>> list_of_objects(DBLP, subclasses)
    [<class 'gismap.sources.dblp.DBLP'>]
    >>> list_of_objects('hal', subclasses)
    [<class 'gismap.sources.hal.HAL'>]
    """
    if default is None:
        default = []
    if clss is None:
        return default
    elif isinstance(clss, str):
        return [dico[clss]]
    elif isinstance(clss, list):
        return [cls for lcls in clss for cls in list_of_objects(lcls, dico, default)]
    else:
        return [clss]



from string import Template
import uuid
import json


vis_template = Template("""
<div class="gismap-content">
<div id="${container_id}"></div>
<a
  href="https://balouf.github.io/gismap/"
  target="_blank"
  id="gismap-brand"
  style="position: absolute; left: 10px; bottom: 10px; text-decoration: none; color: #888; font-size: min(2vw, 10px);
  z-index: 10; pointer-events: auto;"
>
  &copy; Gismap 2025
</a>
<div id="${modal_id}" class="modal">
<div class="modal-content">
    <span class="close" id="${modal_close_id}">&times;</span>
    <div id="${modal_body_id}"></div>
  </div>
</div>
</div>
<style>
.gismap-content {
position: relative;
width: 100%;
height: 80vh !important;
max-width: 100vw;
max-height: 100vh !important;
}
  /* Styles adaptatifs pour dark/light */
/* Default: dark mode styles */
#${container_id} {
  width: 100%;
  height: 100%;
  box-sizing: border-box;
  border: 1px solid #444;
  background: #181818;
}

.modal {
  display: none;
  position: fixed;
  z-index: 1000;
  left: 0; top: 0;
  width: 100%; height: 100%;
  overflow: auto;
  background-color: rgba(10,10,10,0.85);
}

.modal-content {
  background-color: #23272e;
  color: #f0f0f0;
  margin: 10% auto;
  padding: 24px;
  border: 1px solid #888;
  width: 50%;
  border-radius: 8px;
  box-shadow: 0 5px 15px rgba(0,0,0,.6);
}

.close {
  color: #aaa;
  float: right;
  font-size: 28px;
  font-weight: bold;
  cursor: pointer;
}

.close:hover, .close:focus {
  color: #fff;
  text-decoration: none;
  cursor: pointer;
}

/* PyData Sphinx Light Theme */
html[data-theme="light"] #${container_id},
body[data-jp-theme-light="true"] #${container_id} {
  background: #fff;
  border: 1px solid #ccc;
}

html[data-theme="light"] .modal,
body[data-jp-theme-light="true"] .modal {
  background-color: rgba(220,220,220,0.85);
}

html[data-theme="light"] .modal-content,
body[data-jp-theme-light="true"] .modal-content {
  background: #fff;
  color: #222;
  border: 1px solid #888;
}

html[data-theme="light"] .close,
body[data-jp-theme-light="true"] .close {
  color: #222;
}

html[data-theme="light"] .close:hover, html[data-theme="light"] .close:focus,
body[data-jp-theme-light="true"] .close:hover, body[data-jp-theme-light="true"] .close:focus {
  color: #555;
}

/* Fallback: system light mode */
@media (prefers-color-scheme: light) {
  #${container_id} {
    background: #fff;
    border: 1px solid #ccc;
  }
  .modal {
    background-color: rgba(220,220,220,0.85);
  }
  .modal-content {
    background: #fff;
    color: #222;
    border: 1px solid #888;
  }
  .close {
    color: #222;
  }
  .close:hover, .close:focus {
    color: #555;
  }
}
</style>
<script type="text/javascript">
(function() {
  // Détection du thème
function getTheme() {
  // Try PyData Sphinx theme on <html>
  const pydataTheme = document.documentElement.getAttribute("data-theme");
  if (pydataTheme === "dark" || pydataTheme === "light") {
    return pydataTheme;
  }

  // Try JupyterLab theme on <body>
  const jupyterLabTheme = document.body.getAttribute("data-jp-theme-name");
  if (jupyterLabTheme) {
    // Simplify theme name to 'dark' or 'light'
    const lowerName = jupyterLabTheme.toLowerCase();
    if (lowerName.includes("dark")) {
      return "dark";
    }
    if (lowerName.includes("light")) {
      return "light";
    }
  }

  // Fallback to system preference
  return window.matchMedia && window.matchMedia('(prefers-color-scheme: dark)').matches ? "dark" : "light";
};
  function getVisOptions(theme) {
    if (theme === 'dark') {
      return {
        nodes: {
          shape: 'circle', size: 20,
          font: { size: 16, color: '#f0f0f0' },
          color: {
            background: '#222e3c',
            border: '#5d90f5',
            highlight: { background: '#2f3d4d', border: '#f5a25d' }
          },
          borderWidth: 2
        },
        edges: {
          width: 2,
          color: { color: '#888', highlight: '#f5a25d' },
          smooth: { type: 'continuous' }
        },
        interaction: { hover: true }
      };
    } else {
      return {
        nodes: {
          shape: 'circle', size: 20,
          font: { size: 16, color: '#222' },
          color: {
            background: '#e3eaff',
            border: '#3d6cf7',
            highlight: { background: '#fffbe6', border: '#f5a25d' }
          },
          borderWidth: 2
        },
        edges: {
          width: 2,
          color: { color: '#848484', highlight: '#f5a25d' },
          smooth: { type: 'continuous' }
        },
        interaction: { hover: true }
      };
    }
  };

  var physics = {
  physics: {
    solver: "forceAtlas2Based",
    forceAtlas2Based: {
      gravitationalConstant: -50,
      centralGravity: 0.01,
      springLength: 200,
      springConstant: 0.08,
      damping: 0.98,
      avoidOverlap: 1
    },
    maxVelocity: 10,
    minVelocity: 0.9,
    stabilization: {
      enabled: true,
      iterations: 2000,
      updateInterval: 50,
      onlyDynamicEdges: false,
      fit: true
    },
    timestep: 0.25
  }
};

  function renderNetwork() {
  const nodes = new vis.DataSet(${nodes_json});
  const edges = new vis.DataSet(${edges_json});
  const container = document.getElementById('${container_id}');
  let network = null;
    const theme = getTheme();
    const options = getVisOptions(theme);
    network = new vis.Network(container, { nodes: nodes, edges: edges }, options);
    network.setOptions(physics)
    // Tooltip survol
    network.on("hoverNode", function(params) {
      const node = nodes.get(params.node);
      network.body.container.title = node.hover || '';
    });
    network.on("blurNode", function(params) {
      network.body.container.title = '';
    });
    network.on("hoverEdge", function(params) {
      const edge = edges.get(params.edge);
      network.body.container.title = edge.hover || '';
    });
    network.on("blurEdge", function(params) {
      network.body.container.title = '';
    });
    // Modal overlay
    const modal = document.getElementById('${modal_id}');
    const modalBody = document.getElementById('${modal_body_id}');
    const modalClose = document.getElementById('${modal_close_id}');
    network.on("click", function(params) {
      if (params.nodes.length === 1) {
        const node = nodes.get(params.nodes[0]);
        modalBody.innerHTML = node.overlay || '';
        modal.style.display = "block";
      } else if (params.edges.length === 1) {
        const edge = edges.get(params.edges[0]);
        modalBody.innerHTML = edge.overlay || '';
        modal.style.display = "block";
      } else {
        modal.style.display = "none";
      }
    });
    modalClose.onclick = function() { modal.style.display = "none"; };
    window.onclick = function(event) {
      if (event.target == modal) { modal.style.display = "none"; }
    };
  };

  function loadVisAndRender() {
  if (typeof vis === 'undefined') {
    var script = document.createElement('script');
    script.src = "https://unpkg.com/vis-network/standalone/umd/vis-network.min.js";
    script.type = "text/javascript";
    script.onload = function() {
      console.log("vis-network loaded dynamically");
      renderNetwork();  // Graph init after vis is loaded
    };
    document.head.appendChild(script);
  } else {
    console.log("vis-network already loaded");
    renderNetwork();  // Graph init immediately
  }
}
loadVisAndRender();

  // Adapter dynamiquement si le thème change
  window.addEventListener("theme-changed", () => loadVisAndRender());
  const observer = new MutationObserver(mutations => {
  for (const mutation of mutations) {
    if (mutation.type === "attributes" && mutation.attributeName === "data-jp-theme-name") {
      loadVisAndRender();
    }
  }
    });
    observer.observe(document.body, { attributes: true });
    if (window.matchMedia) {
      window.matchMedia('(prefers-color-scheme: dark)').addEventListener('change', () => loadVisAndRender());
    };
})();
</script>
""")


def generate_html(nodes, edges):
    """
    Parameters
    ----------
    nodes: :class:`list`
    edges: :class:`list`

    Returns
    -------
    :class:`str`
    """
    uid = str(uuid.uuid4())[:8]  # identifiant unique pour éviter les collisions
    container_id = f"mynetwork_{uid}"
    modal_id = f"modal_{uid}"
    modal_body_id = f"modal_body_{uid}"
    modal_close_id = f"modal_close_{uid}"
    nodes_json = json.dumps(nodes)
    edges_json = json.dumps(edges)
    dico = {
        "container_id": container_id,
        "modal_id": modal_id,
        "modal_body_id": modal_body_id,
        "modal_close_id": modal_close_id,
        "nodes_json": nodes_json,
        "edges_json": edges_json,
    }
    return vis_template.substitute(dico)  # html_template



from gismap.lab.lab import (
    Lab as Lab,
    ListLab as ListLab,
)
from gismap.lab.lip6 import Lip6 as Lip6, Lip6Lab as Lip6Lab
from gismap.lab.toulouse import Solace as Solace, LaasLab as LaasLab
from gismap.lab.graph import lab2graph as lab2graph
from gismap.lab.vis import generate_html as generate_html
from gismap.lab.egomap import EgoMap as EgoMap
from gismap.lab.lab_author import LabAuthor as LabAuthor



from dataclasses import dataclass, field

from gismap import get_classes, HAL, DBLP
from gismap.sources.models import DB
from gismap.sources.multi import SourcedAuthor, sort_author_sources
from gismap.utils.common import LazyRepr, list_of_objects
from gismap.utils.logger import logger

db_dict = get_classes(DB, key="db_name")
default_dbs = [HAL, DBLP]


@dataclass(repr=False)
class AuthorMetadata(LazyRepr):
    """
    Optional information about an author to be used to enhance her presentation.

    Attributes
    ----------

    url: :class:`str`
        Homepage of the author.
    img: :class:`str`
        Url to a picture.
    group: :class:`str`
        Group of the author.
    position: :class:`tuple`
        Coordinates of the author.
    """

    url: str = None
    img: str = None
    group: str = None
    position: tuple = None


@dataclass(repr=False)
class LabAuthor(SourcedAuthor):
    metadata: AuthorMetadata = field(default_factory=AuthorMetadata)

    def auto_img(self):
        for source in self.sources:
            img = getattr(source, "img", None)
            if img is not None:
                self.metadata.img = img
                break

    def auto_sources(self, dbs=None):
        """
        Automatically populate the sources based on author's name.

        Parameters
        ----------
        dbs: :class:`list`, default=[:class:`~gismap.sources.hal.HAL`, :class:`~gismap.sources.dblp.DBLP`]
            List of DB sources to use.

        Returns
        -------
        None
        """
        dbs = list_of_objects(dbs, db_dict, default=default_dbs)
        sources = []
        for db in dbs:
            source = db.search_author(self.name)
            if len(source) == 0:
                logger.warning(f"{self.name} not found in {db.db_name}")
            elif len(source) > 1:
                logger.warning(f"Multiple entries for {self.name} in {db.db_name}")
            sources += source
        if len(sources) > 0:
            self.sources = sort_author_sources(sources)


def labify_author(author, rosetta):
    if isinstance(author, LabAuthor):
        return author
    return rosetta.get(author.key, author)


def labify_publications(pubs, rosetta):
    for pub in pubs:
        pub.authors = [labify_author(a, rosetta) for a in pub.authors]
        for source in getattr(pub, "sources", []):
            source.authors = [labify_author(a, rosetta) for a in pub.authors]
